# -*- coding: utf-8 -*-
'''Goto - urllib object wrapper'''

__version__ = '0.1.3'


import sys
import types

from . import goto
from . import utils


class CallableModule(types.ModuleType):
    def __init__(self):
        g = globals()
        for name in ['__version__', '__file__', '__package__',
                    '__name__', '__doc__']:
            setattr(self, name, g.get(name))

    __call__ = staticmethod(goto.goto)
    decode_qs = staticmethod(utils.decode_qs)  
    url_fix = staticmethod(goto.url_fix)


sys.modules[__name__] = CallableModule()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import urllib
from cookielib import CookieJar, Cookie


class cached_property(object):
    def __init__(self, f):
        self.f = f

    def __get__(self, instance, owner):
        value = self.f(instance)
        setattr(instance, self.f.__name__, value)
        return value


class cached_classmethod(object):
    def __init__(self, f):
        self.f = f

    def __get__(self, instance, owner):
        value = self.f(owner)
        setattr(owner, self.f.__name__, value)
        return value


class extras(object):
    '''Lazy import extra modules'''
    
    @cached_classmethod
    def json(cls):
        try:
            import simplejson as json
        except ImportError:
            import json
        return json

    @cached_classmethod
    def leaf(cls):
        import leaf
        return leaf

    @cached_classmethod
    def pickle(cls):
        try:
            import cPickle as pickle
        except ImportError:
            import pickle
        return pickle

def encoded_unique_items(items_or_dict, charset='utf-8'):
    if isinstance(items_or_dict, dict):
        items_or_dict = items_or_dict.iteritems()
    ret = {}
    for k, v in items_or_dict:
        ret[(
            k.encode(charset) if isinstance(k, unicode) else k,
            v.encode(charset) if isinstance(v, unicode) else v
        )] = True
    return ret.keys()

def decode_qs(qs, charset='utf-8'):
    '''qs to query dict'''
    import cgi
    return (u'post=(\n%s\n)' % 
        ',\n'.join(["    (%s, u'%s')" % ("'%s'" % k.strip(),
            v[0].strip().decode(charset))
        for k, v in cgi.parse_qs(qs, keep_blank_values=True).items()]
        )).encode('utf-8')

def cj_dumps(cj, dumper='json'):
    return extras.pickle.dumps(cj._cookies) 

def cj_from_dump(dump, dumper='json'):
    cj = CookieJar()
    if isinstance(dump, unicode):
        dump = dump.encode('utf-8')
    cj._cookies = extras.pickle.loads(dump)
    return cj

def cj_from_dict(cookies_dict, req):
    cj = CookieJar()
    for name, value in cookies_dict.iteritems():
        cookie = Cookie(
            version=0,
            name=name,
            value=value,
            port=None,
            port_specified=False,
            domain=req.get_host(),
            domain_specified=False,
            domain_initial_dot=False,
            path='/',
            path_specified=True,
            secure=False,
            expires=None,
            discard=True,
            comment=None,
            comment_url=None,
            rest={'HttpOnly': None},
            rfc2109=False
        )
        cj.set_cookie(cookie)
    return cj

def cj_from(cookies, req):
    if isinstance(cookies, CookieJar):
        return cookies
    if cookies is None:
        return CookieJar()
    if isinstance(cookies, dict):
        return cj_from_dict(cookies, req)
    if isinstance(cookies, basestring):
        return cj_from_dump(cookies)
    assert False, 'cookies must be StringCookieJar, dict or basestring type'

def url_fix(url, charset='utf-8'):
    '''
        Fix no path:
            >>> url_fix('host.com')
            'http://host.com/'
            
            >>> url_fix('host.com?a=b')
            'http://host.com/?a=b'
            
            >>> url_fix('host.com#a=b')
            'http://host.com/#a=b'
            
            >>> url_fix('https://host.com?a=b&a=c#id')
            'https://host.com/?a=b&a=c#id'
            
        Fix spaces:
            >>> url_fix('en.wikipedia.org/wiki/Doorway page')
            'http://en.wikipedia.org/wiki/Doorway%20page'
            
        Fix unicode:
            >>> url_fix('ru.wikipedia.org/wiki/Питон'.decode('utf-8'))
            'http://ru.wikipedia.org/wiki/%D0%9F%D0%B8%D1%82%D0%BE%D0%BD'
        
        Fix idna host:
            >>> url_fix('рег.рф/aaa/'.decode('utf-8'))
            'http://xn--c1ad6a.xn--p1ai/aaa/'
    '''
    url = url.strip()
    if not url.startswith(('http://', 'https://', 'ftp://', 'ftps://')):
        # no schema
        url = 'http://' + url
    if '/' not in url.split('//')[-1]:
        # no path
        frags = [url.find(sep) for sep in ('?', '#') if sep in url]
        if frags:
            x = min(frags)
            host, qs = url[:x], url[x:]
        else:
            host, qs = url, ''
        url = host + '/' + qs
    if isinstance(url, unicode):
        url = url.encode(charset)
    host = url.split('//', 1)[1].split('/', 1)[0]
    if not host.replace('.', '').replace('-', '').isalnum():
        # idn
        host = host.decode('utf-8').encode('idna')
        schema = url.split('//', 1)[0]
        uri = url.split('/', 3)[-1]
        url = '%s//%s/%s' % (schema, host, uri)
    url = urllib.quote(url, safe="%/:=&?~#+!$,;'@()*[]")
    return url




import random

class RNDDict(object):
    TPL = None
    DATA = {}
    
    def __getitem__(self, key):
        return random.choice(self.DATA[key])

    def __str__(self):
        return self.TPL % self


class UAOpera(RNDDict):
    # http://www.useragentstring.com/pages/Opera/
    TPL = '%(name)s (%(os)s; U; %(lang)s) Presto/%(presto)s Version/%(ver)s'
    DATA = {
        'name': ['Opera/9.80'],
        'os': ['Windows NT 6.1', 'Windows NT 6.0', 'Windows NT 5.1'],
        'lang': ['en', 'en-US', 'ru', 'de'],
        'presto': ['2.8.99', '2.8.131', '2.7.62'],
        'ver': ['11.10', '11.01'],
    }

class UAFirefox(RNDDict):
    TPL = '%(name)s (Windows; U; %(os)s; %(lang)s) Gecko/%(engine)s Firefox/%(ver)s'
    DATA = {
        'name': ['Mozilla/5.0'],
        'os': ['Windows NT 6.1', 'Windows NT 6.0', 'Windows NT 5.1'],
        'lang': ['en', 'en-US', 'ru', 'de'],
        'engine': ['20100401', '20091204', '20090402', '20091124', '20100824', '20100722'],
        'ver': ['4.0', '3.8', '3.6.9', '3.6.8', '3.6.7', '3.6.6', '3.6.5', '3.6.3', '3.6.2', '3.6.16', '3.6.15', '3.6.14', '3.6.13', '3.6.12',],
        'rv': ['1.9.2.8', '1.9.2.3', '1.9.2.9', '', '', ],
    }

class UAChrome(RNDDict):
    TPL = '%(name)s (%(os)s) AppleWebKit/%(engine)s (KHTML, like Gecko) Chrome/%(ver)s'
    DATA = {
        'name': ['Mozilla/5.0'],
        'os': ['Windows NT 6.1', 'Windows NT 6.0', 'Windows NT 5.1'],
        'engine': ['534.25', '534.24'],
        'ver': ['12.0.706.0 Safari/534.25', '12.0.702.0 Safari/534.24', '11.0.699.0 Safari/534.24', '11.0.697.0 Safari/534.24'],
    }

class UAIE8(RNDDict):
    TPL = '%(ua)s'
    DATA = {
        'ua': [s.strip() for s in '''
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; Media Center PC 4.0; SLCC1; .NET CLR 3.0.04320)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 1.1.4322)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; InfoPath.2; SLCC1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 2.0.50727)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727)
            Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 5.0; Trident/4.0; InfoPath.1; SV1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; .NET CLR 3.0.04506.30)
            Mozilla/5.0 (compatible; MSIE 7.0; Windows NT 5.0; Trident/4.0; FBSMTWB; .NET CLR 2.0.34861; .NET CLR 3.0.3746.3218; .NET CLR 3.5.33652; msn OptimizedIE8;ENUS)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.2; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; Media Center PC 6.0; InfoPath.2; MS-RTC LM 8
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; Media Center PC 6.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET4.0C)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; InfoPath.3; .NET4.0C; .NET4.0E; .NET CLR 3.5.30729; .NET CLR 3.0.30729; MS-RTC LM 8)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; InfoPath.2)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; Zune 3.0)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; msn OptimizedIE8;ZHCN)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; InfoPath.3; .NET4.0C; .NET4.0E) chromeframe/8.0.552.224
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; .NET4.0C; .NET4.0E; Zune 4.7; InfoPath.3)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8; .NET4.0C; .NET4.0E; Zune 4.7)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MS-RTC LM 8)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; Zune 4.0)
            Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; MS-RTC LM 8; Zune 4.7)
        '''.split('\n') if s.strip()],
    }

USER_AGENTS = [UAOpera, UAFirefox, UAChrome, UAIE8]




#!/usr/bin/env python
from setuptools import setup
import goto

setup(
    name='goto',
    version=goto.__version__,
    description='Urllib object wrapper',
    long_description=open('README.rst').read(),
    author='Imbolc',
    author_email='imbolc@imbolc.name',
    url='https://bitbucket.org/imbolc/goto',
    packages= ['goto', ],
    install_requires=[],
    license='ISC',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
    ),
)
